(ns wanijo.forms
  (:require [hiccup.form :as hform]
            [clojure.spec.alpha :as spec]
            [wanijo.view :as view]))

(defn spec-to-errmsg [label spec-key field-value]
  (view/flash-error
   (map
    (fn [prob]
      [:p
       "Field "
       [:span.flash__field label]
       " must comply to "
       [:span.flash__pred (:pred prob)]])
    (:clojure.spec.alpha/problems
     (spec/explain-data spec-key field-value)))))

(defn field-valid? [value spec-key req]
  (or (empty? (:form-params req))
      (spec/valid? spec-key value)))

(defn field [form-def field req]
  (let [field-value (get-in req [:params field])
        field-def (get-in form-def [:fields field])
        {:keys [label required] spec-key :spec} field-def]
    (list
     (when-not (field-valid? field-value spec-key req)
       (spec-to-errmsg label spec-key field-value))
     (hform/label field label)
     (hform/text-field {:required (when required "required")}
                       field
                       field-value))))

(defn valid? [form-def req]
  (clojure.pprint/pprint req)
  (reduce-kv
   (fn [result field value]
     (if (spec/valid?
          (get-in form-def [:fields field :spec])
          value)
       true
       (reduced false)))
   true
   (:params req)))
